# -*- encoding: utf-8 -*-
##################################################################################
#                                                                                #
#    Copyright (C) 2011 Synkronized (<http://synkronized.be>).                   #
#                                                                                #
#    This program is free software: you can redistribute it and/or modify        #
#    it under the terms of the GNU Affero General Public License as              #
#    published by the Free Software Foundation, either version 3 of the          #
#    License, or (at your option) any later version.                             #
#                                                                                #
#    This program is distributed in the hope that it will be useful,             #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of              #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
#    GNU Affero General Public License for more details.                         #
#                                                                                #
#    You should have received a copy of the GNU Affero General Public License    #
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                                #
##################################################################################

from osv import osv, fields

class account_invoice(osv.osv):
    _name = 'account.invoice'
    _inherit = 'account.invoice'
    
    def copy(self, cr, uid, ids, defaults, context=None):
        defaults['communication'] = ''
        return super(account_invoice, self).copy(cr, uid, ids, defaults, context=context)

    _columns = {
        'communication': fields.char("Communication", size=12, readonly=True)
    }
    _sql_constraints = [
        ('communication_uniq', 'unique (communication)', "The communication must be unique !")
        ]
    
    def action_number(self, cr, uid, ids, context=None):        
        result = super(account_invoice, self).action_number(cr, uid, ids)
        
        if result:
            for invoice in self.browse(cr, uid, ids, context=context):
                
                tries = 5
                ref = invoice.id
                ref_str = ''
                while tries > 0:
                    try:
                        ref_base = ("%010d" % ref)[::-1]
                        checksum = "%02d" % ((int(ref_base) % 97) or 97)
                        ref_str = ref_base + checksum
                        self.write(cr, uid, [invoice.id], {'communication': ref_str})
                        break
                    except:
                        ref += 1
                        tries -= 1
                                                
                # update relevant account_move_line of type receivable
                account_ids = self.pool.get('account.account').search(cr, uid, [('type','=','receivable')])
                cr.execute('UPDATE account_move_line SET ref=%s ' \
                        'WHERE move_id=%s AND account_id in ('+','.join(map(str,account_ids))+')', (ref_str, invoice.move_id.id))
            
        return result
    
account_invoice()

class account_bank_statement(osv.osv):
    _name = 'account.bank.statement'
    _inherit = 'account.bank.statement'
    
    def reconcile_using_communication(self, cr, uid, ids, context=None):
        invoice_ref = self.pool.get('account.invoice')

        for statement in self.browse(cr, uid, ids, context=context):
            for line in statement.line_ids:
                if line.voucher_id:
                    continue

                invoice_ids = invoice_ref.search(cr, uid, [('communication','=',line.name)])

                if not len(invoice_ids) == 1:
                    continue

                invoice = invoice_ref.browse(cr, uid, invoice_ids, context=context)[0]
                line.write({'voucher_id': line.voucher_for_invoice(invoice, statement.journal_id.id, context=context)})

        return True

account_bank_statement()

class account_bank_statement_line(osv.osv):
    _name = 'account.bank.statement.line'
    _inherit = 'account.bank.statement.line'

    def voucher_for_invoice(self, cr, uid, ids, invoice, journal_id, context=None):
        # search invoice and create voucher

        account_voucher_ref = self.pool.get('account.voucher')
        vd = account_voucher_ref.onchange_partner_id(cr, uid, [], invoice.partner_id.id, journal_id, invoice.amount_total, None, 'receipt', None, context=context)['value']

        vd['partner_id'] = invoice.partner_id.id
        vd['amount'] = invoice.amount_total
        vd['line_ids'] = []
        vd['line_dr_ids'] = []
        vd['line_cr_ids'] = [(0,0,t) for t in vd['line_cr_ids'] if t['move_line_id'] in [ml.id for ml in invoice.move_id.line_id]]

        return account_voucher_ref.create(cr, uid, vd)

    def onchange_name(self, cr, uid, ids, name, journal_id, context=None):
        if not name:
            return {}

        invoice_ref = self.pool.get('account.invoice')
        invoice_ids = invoice_ref.search(cr, uid, [('communication','=',name)])

        if not len(invoice_ids) == 1:
            return {}

        invoice = invoice_ref.browse(cr, uid, invoice_ids, context=context)[0]

        d = {
            'partner_id': invoice.partner_id.id,
            'amount': invoice.amount_total
             }
        d.update(self.onchange_partner_id(cr, uid, ids, invoice.partner_id.id, context=context)['value'])
        d['voucher_id'] = self.voucher_for_invoice(cr, uid, ids, invoice, journal_id, context=context)
        
        return {'value': d}

    def match_invoices(self, cr, uid, ids, context=None):
        
        return {}

account_bank_statement_line()

